/*
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/


/*****   TIDA-00480 Automotive Hall Sensor Rotary Encoder Software v1.0   *****/
/*****                             main.c                                 *****/
/*****                      Author: Nikkan Yadegary                       *****/


#include <msp430.h> 

/* D0 7SEG LED display digit value */
unsigned char D0OUT;

/* D1 7SEG LED display digit value */
unsigned char D1OUT;

/* D2 7SEG LED display digit value */
unsigned char D2OUT;

/* U3 Hall Sensor device output reading (previous state) */
unsigned char U3PREV;

/* U4 Hall Sensor device output reading (previous state) */
unsigned char U4PREV;

/* Rotation in the CW (1) or CCW (0) direction */
unsigned char CWDIR;

void main(void)
{
	/* Stop watchdog timer */
    WDTCTL = WDTPW | WDTHOLD;
	
	/* 1MHz Clock Speed */
	BCSCTL1 = CALBC1_1MHZ;
	DCOCTL  = CALDCO_1MHZ;

	/* SMCLK/1 */
	BCSCTL2 |= DIVS_0;

	/* LFXT1 = VLO, Crystal Capacitance 12.5pF */
	BCSCTL3 |= (LFXT1S_2 + XCAP_3);

	/* Initialize all Port 1 and Port 2 pins to GPIO functionality */
	P1SEL  = 0;
	P1SEL2 = 0;
	P2SEL  = 0;
	P2SEL2 = 0;

	/* Set unused outputs low to reduce power consumption */
	P1OUT &= ~(BIT4 + BIT5 + BIT6 + BIT7);
	P1DIR |= BIT4 + BIT5 + BIT6 + BIT7;

	P2OUT &= ~(BIT5 + BIT6 + BIT7);
	P2DIR |= BIT5 + BIT6 + BIT7;

	/* Initialize D0, D1, and D2 7SEG LED displays to 000 */
	P1OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3);
	P1DIR |= BIT0 + BIT1 + BIT2 + BIT3;

	P2OUT &= ~(BIT0 + BIT1 + BIT2);
	P2DIR |= BIT0 + BIT1 + BIT2;

	P2OUT |= BIT0 + BIT1 + BIT2;

	/* Initialize 3-digit count to 000 */
	D0OUT = 0;
	D1OUT = 0;
	D2OUT = 0;

	/* Store initial Hall Sensor output readings */
	U3PREV = (P2IN & BIT3);
	U4PREV = (P2IN & BIT4);

	while(1)
	{
		/* Check to see if the current Hall Sensor output readings have changed from previous readings */
		if ((((P2IN & BIT3) != U3PREV) || ((P2IN & BIT4) != U4PREV)))
		{
			/* Determine CW or CCW direction by comparing current/previous Hall Sensor output readings */
			if ((((P2IN & BIT3) == 0) && ((P2IN & BIT4) == 0)))
			{
				if (U3PREV == 8)
				{
					CWDIR = 1;
				}
				else
				{
					CWDIR = 0;
				}
			}
			else if ((((P2IN & BIT3) == 0) && ((P2IN & BIT4) == 16)))
			{
				if (U3PREV == 0)
				{
					CWDIR = 1;
				}
				else
				{
					CWDIR = 0;
				}
			}
			else if ((((P2IN & BIT3) == 8) && ((P2IN & BIT4) == 16)))
			{
				if (U3PREV == 0)
				{
					CWDIR = 1;
				}
				else
				{
					CWDIR = 0;
				}
			}
			else if ((((P2IN & BIT3) == 8) && ((P2IN & BIT4) == 0)))
			{
				if (U3PREV == 8)
				{
					CWDIR = 1;
				}
				else
				{
					CWDIR = 0;
				}
			}

			/* Increase/Decrease 3-digit count by 1 for rotation in the CW/CCW direction */
			if (CWDIR == 1)
			{
				if (D0OUT < 9)
				{
					D0OUT++;
				}
				else if (D0OUT == 9)
				{
					if (D1OUT < 9)
					{
						D0OUT = 0;
						D1OUT++;
					}
					else if (D1OUT == 9)
					{
						if (D2OUT < 9)
						{
							D0OUT = 0;
							D1OUT = 0;
							D2OUT++;
						}
					}
				}
			}
			else if (CWDIR == 0)
			{
				if (D0OUT > 0)
				{
					D0OUT--;
				}
				else if (D0OUT == 0)
				{
					if (D1OUT > 0)
					{
						D0OUT = 9;
						D1OUT--;
					}
					else if (D1OUT == 0)
					{
						if (D2OUT > 0)
						{
							D0OUT = 9;
							D1OUT = 9;
							D2OUT--;
						}
					}
				}
			}

			/* Update D0 7SEG LED display according to count */
			P1OUT = D0OUT;
			P2OUT &= ~BIT0;
			P2OUT |= BIT0;

			/* Update D1 7SEG LED display according to count */
			P1OUT = D1OUT;
			P2OUT &= ~BIT1;
			P2OUT |= BIT1;

			/* Update D2 7SEG LED display according to count */
			P1OUT = D2OUT;
			P2OUT &= ~BIT2;
			P2OUT |= BIT2;

			/* Store Hall Sensor output readings for the next rotation event */
			U3PREV = (P2IN & BIT3);
			U4PREV = (P2IN & BIT4);
		}
	}
}

#pragma vector=PORT1_VECTOR,		\
	PORT2_VECTOR,					\
	TIMER0_A0_VECTOR,               \
	TIMER0_A1_VECTOR,               \
	TIMER1_A0_VECTOR,               \
	TIMER1_A1_VECTOR,               \
	USCIAB0RX_VECTOR,               \
	USCIAB0TX_VECTOR,               \
	WDT_VECTOR,               		\
	NMI_VECTOR,						\
	COMPARATORA_VECTOR,         	\
	ADC10_VECTOR
__interrupt void ISR_trap(void)
{
	/* The following will cause an access violation which results in a PUC reset */
	WDTCTL = 0;
}
